@@ -4,7 +4,7 @@
  *
  *    Graphics Subsystem interface
  *
- *  Copyright 1999, 2000, 2001, 2002
+ *  Copyright 1999, 2000, 2001, 2002, 2003, 2004, 2005
  *     - The FreeType Development Team - www.freetype.org
  *
  ***************************************************************************/
@@ -29,12 +29,6 @@
   /* define the global error variable */
   extern int  grError;
 
-  /* initialisation */
-  extern int  grInit( void );
-
-  /* finalisation */
-  extern void grDone( void );
-
 
   /* pixel mode constants */
   typedef enum grPixelMode
@@ -50,6 +44,8 @@
     gr_pixel_mode_rgb32,       /* 32-bits mode - 16 million colors */
     gr_pixel_mode_lcd,         /* horizontal RGB-decimated         */
     gr_pixel_mode_lcdv,        /* vertical RGB-decimated           */
+    gr_pixel_mode_lcd2,        /* horizontal BGR-decimated         */
+    gr_pixel_mode_lcdv2,       /* vertical BGR-decimated           */
 
     gr_pixel_mode_max          /* don't remove */
 
@@ -163,7 +159,6 @@
   *    writes a given glyph bitmap to a target surface.
   *
   * <Input>
-  *    is_bgr  :: 1 if BGR, 0 if RGB
   *    target  :: handle to target bitmap
   *    glyph   :: handle to source glyph bitmap
   *    x       :: position of left-most pixel of glyph image in target surface
@@ -188,14 +183,15 @@
   **********************************************************************/
 
   extern int
-  grBlitGlyphToBitmap( int        is_bgr,
-                       grBitmap*  target,
+  grBlitGlyphToBitmap( grBitmap*  target,
                        grBitmap*  glyph,
                        grPos      x,
                        grPos      y,
                        grColor    color );
 
 
+#if 0
+
  /**********************************************************************
   *
   * <Function>
@@ -221,6 +217,7 @@
                                 grPos      height,
                                 grColor    color );
 
+#endif /* 0 */
 
 
  /**********************************************************************
@@ -380,6 +377,19 @@
   grDeviceChain*  grInitDevices( void );
 
 
+ /**********************************************************************
+  *
+  * <Function>
+  *    grDoneDevices
+  *
+  * <Description>
+  *    Finalize all devices activated with grInitDevices.
+  *
+  **********************************************************************/
+
+  extern
+  void  grDoneDevices( void );
+
 
  /**********************************************************************
   *
@@ -476,7 +486,7 @@
   extern grSurface*  grNewSurface( const char*  device,
                                    grBitmap*    bitmap );
 
-
+  extern void  grDoneSurface( grSurface*  surface );
 
  /**********************************************************************
   *
@@ -631,5 +641,23 @@
                          int         event_mask,
                          grEvent    *event );
 
+ /**********************************************************************
+  *
+  * <Function>
+  *    grSetGlyphGamma
+  *
+  * <Description>
+  *    set the gamma-correction coefficient. This is only used to
+  *    blit glyphs
+  *
+  * <Input>
+  *    gamma      :: gamma value. <= 0 to select sRGB transfer function
+  *
+  **********************************************************************/
+
+  extern
+  void  grSetGlyphGamma( double  gamma_value );
+
+/* */
 
 #endif /* GRAPH_H */
